BODY{ background: url(bg1.gif) }

A:link {text-decoration:none; color: #004080}
A:visited {text-decoration:none; color: #003060}
A:active {text-decoration:none; color:#6F6F00}


TD { font-family:Arial,sans-serif }
TD.outside{background-color: #D0A000} 
TD.inside{background-color: #FAF8D0; text-align: center; font-size:14px;} 
TD.selected_outside{background-color: #0000E0}
TD.alt_outside{background-color:#88BBAA}
